function [fgz, fmu, fmuMix] = FunAFTER(z,id0,y,X,lam,subjectTt,Tt,Family,CorrStruct)
% Evaluating AFTER or AFTQR type forecast for conditional density function and conditional mean up to time Tt
% fgz is the AFTER forecast of conditional density function at z = 0 or 1
% fmu is the AFTER forecast of conditional mean
% fmuMix is the AFTQR forecast of conditional mean
% id0 is the subject id corresponding label of the subject you want to forecast, id0 \in\{ 1,2,...,n\}
%  y = vector of binary or count responses
%  X = Matrix of covariates 
% lam = lambda in the AFTQR weight 
% subject = the vector of labels for repeated measures in panel data
% Family = Assumed distribution family of y
% Family = 
%               2 | b: binary outcome or Bernoulli
%               3 | p: counts or Poisson
% CorrStruct = the labels of correlation structure, in our case study we take CorrStruct=[1;3]  
[N, p]  = size(X);
[~,n]   = subCounti(subjectTt);
[ss,SS] = subChooseVariable(1,p);
Sbb     = length(ss);
Sv      = length(CorrStruct);
J       = Sbb*Sv;
time    = subjectTt;
%% t=1
[idtrain,~]          = subFunPartition(subjectTt,Tt,1);
Xtrain               = X(idtrain,:);
ytrain               = y(idtrain);
subjectTrain         = subjectTt(idtrain);
timeTrain            = time(idtrain);
for j1 = 1:Sbb
     j1label  = SS(j1,(1:ss(j1)));
     Xs1train = Xtrain(:,j1label);
     for j2 = 1:Sv
         corrStruct = CorrStruct(j2);
         bhat       = subFunEstLongGEE(subjectTrain,timeTrain,ytrain,Xs1train,Family,corrStruct);
         for i = 1:n
             xi1    = X((i-1)*Tt+1,j1label);
             switch lower(Family)
                 case {2,'b'}
                     pi1    = exp(xi1*bhat)/(1 + exp(xi1*bhat));
                     yi1    = y((i-1)*Tt+1);
                     gi1    = ((pi1/(1-pi1))^yi1)*(1-pi1);
                     Gt((i-1)*Tt+1,(j1-1)*Sv+j2) = gi1;
                     CDFt((i-1)*Tt+1,(j1-1)*Sv+j2) = binocdf(yi1,1,pi1);
                     E((i-1)*Tt+1,(j1-1)*Sv+j2)  = 1;
                     Q((i-1)*Tt+1,(j1-1)*Sv+j2)  = 1;
                     Mu((i-1)*Tt+1,(j1-1)*Sv+j2) = pi1;
                 case {3,'p'}
                     pi1    = exp(xi1*bhat);
                     yi1    = y((i-1)*Tt+1);
                     gi1    = poisspdf(yi1,pi1);
                     Gt((i-1)*Tt+1,(j1-1)*Sv+j2) = gi1;
                     CDFt((i-1)*Tt+1,(j1-1)*Sv+j2) = poisscdf(yi1,pi1);
                     E((i-1)*Tt+1,(j1-1)*Sv+j2)  = 1;
                     Q((i-1)*Tt+1,(j1-1)*Sv+j2)  = 1;
                     Mu((i-1)*Tt+1,(j1-1)*Sv+j2) = pi1;
                 otherwise
                     disp('ERROR: Invalid distribution family specified');
                     return;
             end
         end
     end
end
%% t\ge 2
for t = 2:Tt
    [idtrain,~]          = subFunPartition(subjectTt,Tt,t-1);
    Xtrain               = X(idtrain,:);
    ytrain               = y(idtrain);
    subjectTrain         = subjectTt(idtrain);
    timeTrain            = time(idtrain);
    for j1 = 1:Sbb
        j1label  = SS(j1,(1:ss(j1)));
        Xsttrain = Xtrain(:,j1label);
        for j2 = 1:Sv
            corrStruct    = CorrStruct(j2);
            [bhat,alphat] = subFunEstLongGEE(subjectTrain,timeTrain,ytrain,Xsttrain,Family,corrStruct);
       switch lower(corrStruct)
       case {1,'ar1'}
         for l=1:t
             for s=1:t
                 Ri1(l,s) = alphat^(abs(l-s));
             end
         end
       case {3,'equi'}
           Ri1 = alphat*ones(t,t)+(1-alphat)*eye(t);
       case {4,'tri'}
           Ri1 = eye(t);
           for s = 2:t
               Ri1(s,s-1) = alphat;
           end
           for l = 1:t-1
               Ri1(l,l+1) = alphat;
           end
       case {6,'ind'}
           Ri1 = eye(t);
       end
               for i = 1:n
                   xit1   = X((((i-1)*Tt+1):((i-1)*Tt+t-1)),j1label);
                   xit    = X((i-1)*Tt+t,j1label);
                   switch lower(Family)
                       case {2,'b'}
                           pit    = exp(xit*bhat)/(1 + exp(xit*bhat));
                           pit1   = exp(xit1*bhat)./(1 + exp(xit1*bhat));
                           pitt   = [pit1;pit];
                           Aitsq  = diag(sqrt(pitt.*(1-pitt)));
                           Vit    = Aitsq*Ri1*Aitsq;
                           sit    = Vit((1:t-1),t);
                           Vit1   = Vit((1:t-1),(1:t-1));
                           kit    = Vit1\sit;
                           yit    = y((i-1)*Tt+t);
                           yi_t1  = y(((i-1)*Tt+1):((i-1)*Tt+t-1));
                           muit   = pit +  kit'*(yi_t1 - pit1);
                           Mu((i-1)*Tt+t,(j1-1)*Sv+j2)    = muit;
                           git                            = ((muit/(1-muit))^yit)*(1-muit);
                           Gt((i-1)*Tt+t,(j1-1)*Sv+j2)     = git;
                           CDFt((i-1)*Tt+t,(j1-1)*Sv+j2)   = binocdf(yit,1,muit);
                           E((i-1)*Tt+t,(j1-1)*Sv+j2)      = git*E(((i-1)*Tt+t-1),(j1-1)*Sv+j2);
                           Q((i-1)*Tt+t,(j1-1)*Sv+j2)     = exp(-lam*(yit - muit)^2)*Q(((i-1)*Tt+t-1),(j1-1)*Sv+j2);
                       case {3,'p'}
                           pit    = exp(xit*bhat);
                           pit1   = exp(xit1*bhat);
                           pitt   = [pit1;pit];
                           Aitsq  = diag(sqrt(pitt));
                           Vit    = Aitsq*Ri1*Aitsq;
                           sit    = Vit((1:t-1),t);
                           Vit1   = Vit((1:t-1),(1:t-1));
                           kit    = Vit1\sit;
                           yit    = y((i-1)*Tt+t);
                           yi_t1  = y(((i-1)*Tt+1):((i-1)*Tt+t-1));
                           muit   = pit +  kit'*(yi_t1 - pit1);
                           Mu((i-1)*Tt+t,(j1-1)*Sv+j2)    = muit;
                           git                            = poisspdf(yit,muit);%((muit/(1-muit))^yit)*(1-muit);
                           Gt((i-1)*Tt+t,(j1-1)*Sv+j2)     = git;
                           CDFt((i-1)*Tt+t,(j1-1)*Sv+j2)   = poisscdf(yit,muit);%binocdf(yit,1,muit);
                           E((i-1)*Tt+t,(j1-1)*Sv+j2)      = git*E(((i-1)*Tt+t-1),(j1-1)*Sv+j2);
                           Q((i-1)*Tt+t,(j1-1)*Sv+j2)      = exp(-lam*(yit - muit)^2)*Q(((i-1)*Tt+t-1),(j1-1)*Sv+j2);
                       otherwise
                           disp('ERROR: Invalid distribution family specified');
                           return;
                   end
               end
        end
    end
   t
end
%%
ww     = zeros(N,J);
lammix = zeros(N,J);
for i=1:n
    ww((i-1)*Tt+2,:) = ones(1,J)/J;
    lammix((i-1)*Tt+2,:) = ones(1,J)/J;
    for t=3:Tt
        for j1=1:Sbb
            for j2 = 1:Sv
                ww((i-1)*Tt+t,(j1-1)*Sv+j2)     = E((i-1)*Tt+t-1,(j1-1)*Sv+j2)/sum(E((i-1)*Tt+t-1,:));
                lammix((i-1)*Tt+t,(j1-1)*Sv+j2) = Q((i-1)*Tt+t-1,(j1-1)*Sv+j2)/sum(Q((i-1)*Tt+t-1,:));
            end
        end
    end
end
fmuJ        = Mu(id0*Tt,:)';
switch lower(Family)
    case {2,'b'}
        fGJ      = binopdf(z,1,fmuJ);
    case {3,'p'}
        fGJ      = poisspdf(z,fmuJ);
    otherwise
        disp('ERROR: Invalid distribution family specified');
        return;
end
fgz       =  ww(id0*Tt,:)*fGJ;% hat g by AFTER
fmu       =  ww(id0*Tt,:)*fmuJ;% hat mu by AFTER
fmuMix    =  lammix(id0*Tt,:)*fmuJ;% hat mu by Mixing
end

function [s,S] = subChooseVariable(k,m)
% There are m variables but jut the first k of them must be in all
% submodels.
q=m-k;
M=2^q;
pp=zeros(q+1,1);
for i=1:q+1
    pp(i,1)=nchoosek(q,i-1);
end
s=zeros(M,1);
s(1,1)=k;
for i=2:q+1
    temp=sum(s>0);
    s(temp+1:temp+pp(i,1),1)=(i+k-1)*ones(pp(i,1),1);
end
S=zeros(M,m);
for i=1:M
    S(i,1:k)=[1:k];
end
tS=1;
for i=1:q
    Temp=nchoosek([k+1:m],i);
    S(tS+1:tS+size(Temp,1),k+1:k+i)=Temp;
    tS=tS+size(Temp,1);
end
end

function [idtrain,idtest] = subFunPartition(subject,T,t)
% 1,...,t are the training data
% t+1,...,T are the test data
[ni,n] = subCounti(subject);
ni = ni';
nstart = [1; cumsum(ni)+1];
id     = [1:n*T]';
idtrain = zeros(n*t,1);
for k=1:n
idtrain(((k-1)*t+1):(k*t))=[nstart(k):(nstart(k)-1+t)]';
end
idtest=id;
idtest(idtrain)=[];
end
function [bhat,alphat, QICimori, QICpan, flag] = subFunEstLongGEE(id,time,y,X,Family,CorrStruct)
flag = 1;
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% 
% Modified from the code written by: Sarah Ratcliffe. 
% Delete Markove type correlation structure
% set maxit = 500; 
% tol = 1e-10; 
% Based on Stata programs by Justine Shults
% Copywrite 2006
%
% COMMAND:  [beta,alpha,results] = gee(id,y,t,X,Family,CorrStruct,varnames,tol,maxit);
% ACTION:   Computes Quassi Least squares (QLS) estimates for repeated
%           measures data y, measured at times t, on subjects id, regressed
%           on covariates X, assuming data is from a Family distribution 
%           with CorrStruct type of correlation.
%
% INPUTS:  id = Column of unique subject identifiers used to indicate
%               measurements from the same subjects.
%           y = Column of outcome variable measurements corresponding to id.
%           t = Column of measurement times corresponding to y.
%               NB: AT THE MOMENT ASSUMING EQUISPACED DATA.
%           X = Matrix of covariates corresponding to y and t. This must 
%               contain a column of 1's in order to include a constant term 
%               in the model.
%          Family = Assumed distribution of the data.
%               1 | n:       Normal distribution  (default)
%               2 | b:       Bernoulli distribution
%               3 | p:       Poisson
%          CorrStruct = Assumed correlation structure
%               1 | ar1:     AR(1) structure  (default) 
%               3 | equi:    Equicorrelated structure
%               4 | tri:     Tri-diagonal structure
%		        5 | un:	     Unstructured correlation matrix
%		        6 | ind:	 Working independent correlation matrix
%    varnames = Optional cell array containing variable names associated
%               with the variables entered in X. (default = {'1','2',...})
%         tol = Convergence tolerance. (default = 0.00001)
%       maxit = Maximum number of iterations. (default = 100)
%
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

maxit = 500; 
tol = 1e-10; 


warning off MATLAB:divideByZero
warning off MATLAB:fzero:UndeterminedSyntax

% Check dimensions of input variables.
[totobs,ct] = size(id);
if totobs==1;  % Change data from row to column vector;
  id = id';
  totobs=ct;
  ct=1;
end;
if ct~=1  % Not 1 column
  disp('ERROR: id must contain a single column of data listing subject ids');
  return;
end;
[rt,ct] = size(y);
if rt==1;  % Change data from row to column vector;
  y = y';
  rt=ct;
  ct=1;
end;
if ct~=1 | rt~=totobs % Not 1 column of same length as id
  disp('ERROR: y must contain a single column of data from the outcome variable');
  disp('       and must be the same length as id');
  return;
end;
[rt,ct] = size(X);
if rt~=totobs;
  disp('ERROR: X must have same number of rows as length of y');
  return;
end;
[totobs,p] = size(X);

% Determine number of observations per subject.
[ni,n] = subCounti(id);
ni = ni';
nstart = [1; cumsum(ni)+1];
nstart = nstart(1:n);

% Generate starting values for beta = bhat;
switch lower(Family);
case {1,'n'}
    bhat = glmfit(X,y,'normal','identity','on',[],[],'off');	
case {2,'b'};
    bhat = glmfit(X,[y ones(totobs,1)],'binomial','logit','on',[],[],'off');
case {3,'p'};
    bhat = glmfit(X,y,'poisson','log','on',[],[],'off');
otherwise
  disp('ERROR: Invalid distribution family specified');
  return;
end;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Start Stage 1 estimation loop; %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
wtol = 1;
iter = 1;


SS = zeros(max(ni),max(ni),n);
K1p = (totobs-n)-p;

while (wtol>tol)&(iter<=maxit);

  xtb = X*bhat;
  switch lower(Family);
  case {1,'n'}
    z = y - xtb;
    huij = ones(totobs,1);
  case {2,'b'};
    uij = exp(xtb)./(1+exp(xtb));
    huij = sqrt(uij.*(1-uij));
    z = (y - uij)./huij;
  case {3,'p'};
    uij = exp(xtb);
    huij = sqrt(uij);
    z = (y - uij)./huij;
  otherwise
  end;

  zlag = [NaN; z(1:totobs-1,1)];
  zlag(nstart) = NaN;

  % Estimate dispersion paramater, phi;
  phihat = sum(z.^2)/(totobs-p);
  
  % Generate correlation parameter estimate based on specified Correlation Structure;
  % Stage 1 estimates of alpha and correlation matrix, R;
  Z1 = [];
  Z2 = [];
  SS = zeros(totobs,max(ni));

  switch lower(CorrStruct);
  case {1,'ar1'};
    
    alphat = nansum(z.*zlag)/(K1p*phihat);

    for i=1:n;
       nii = ni(i);
       hui = huij(nstart(i):nstart(i)+nii-1,1);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       Xi = X(nstart(i):nstart(i)+nii-1,:);
       Ri = alphat.^(abs(kron([1:nii]',ones(1,nii)) - kron(ones(nii,1),[1:nii])));  % Correlation matrix
       if max(max(isnan(Ri)))+max(max(isinf(Ri)))==1
           Ri = eye(nii);
           flag = 0
       end
       if min(eig(Ri))<=0
           Ri = eye(nii);
           flag = 0
       end
       SSi = chol(inv(Ri));
       Ai = diag(hui);
       Z1 = [Z1; SSi*zi];
       Z2 = [Z2; SSi*Ai*Xi];
       SS(nstart(i):nstart(i)+nii-1,1:nii) = SSi';
    end;
    
   case {2,'markov'};
%     
     alphat = nansum(z.*zlag)/(K1p*phihat);
% 
     for i=1:n;
        nii = ni(i);
        ti = time(nstart(i):nstart(i)+nii-1,1);
        hui = huij(nstart(i):nstart(i)+nii-1,1);
        zi = z(nstart(i):nstart(i)+nii-1,1);
        Xi = X(nstart(i):nstart(i)+nii-1,:);
        Ri = alphat.^(abs(kron(ti,ones(1,nii)) - kron(ones(nii,1),ti')));  % Correlation matrix
       if max(max(isnan(Ri)))+max(max(isinf(Ri)))==1
           Ri = eye(nii);
           flag = 0;
       end
       if min(eig(Ri))<=0
           Ri = eye(nii);
           flag = 0;
       end
        SSi = chol(inv(Ri));
        Ai = diag(hui);
        Z1 = [Z1; SSi*zi];
        Z2 = [Z2; SSi*Ai*Xi];
        SS(nstart(i):nstart(i)+nii-1,1:nii) = SSi';
     end;

  case {3,'equi'};
    maxn = max(ni);
    
    temp1 = 0;
    temp2 = 0;
    for i=1:n;
       nii = ni(i);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       temp = zi * zi';
       temp1 = temp1 + sum(sum(temp)) - sum(diag(temp));
       temp2 = temp2 + nii*(nii-1);
    end;
    alphat = temp1 / (phihat*(temp2-p));

    for i=1:n;
       nii = ni(i);
       hui = huij(nstart(i):nstart(i)+nii-1,1);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       Xi = X(nstart(i):nstart(i)+nii-1,:);
       Ri = alphat*ones(nii,nii)+(1-alphat)*eye(nii);   % Correlation matrix
       if max(max(isnan(Ri)))+max(max(isinf(Ri)))==1
           Ri = eye(nii);
           flag = 0
       end
       if min(eig(Ri))<=0
           Ri = eye(nii);
           flag = 0
       end
       SSi = chol(inv(Ri));
       Ai = diag(hui);
       Z1 = [Z1; SSi*zi];
       Z2 = [Z2; SSi*Ai*Xi];
       SS(nstart(i):nstart(i)+nii-1,1:nii) = SSi';
    end;

  case {4,'tri'};

    alphat = nansum(z.*zlag)/(K1p*phihat);
    for i=1:n;
       nii = ni(i);
       hui = huij(nstart(i):nstart(i)+nii-1,1);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       Xi = X(nstart(i):nstart(i)+nii-1,:);
       temp = ones(nii,1);
       Ri = spdiags([temp,alphat*temp,alphat*temp],[0 1 -1],nii,nii);  % Correlation matrix
       if max(max(isnan(Ri)))+max(max(isinf(Ri)))==1
           Ri = eye(nii);
           flag = 0;
       end
       if min(eig(Ri))<=0
           Ri = eye(nii);
           flag = 0;
       end
       SSi = chol(inv(Ri));
       Ai = diag(hui);
       Z1 = [Z1; SSi*zi];
       Z2 = [Z2; SSi*Ai*Xi];
       SS(nstart(i):nstart(i)+nii-1,1:nii) = SSi';
    end;
    
  case {5,'un'};
    maxn = max(ni);
    alphat = 0;
    rjk = zeros(maxn,maxn);
    for i=1:n;
       nii = ni(i);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       rjk(1:nii,1:nii) = rjk(1:nii,1:nii) + zi*zi';
   end;
   rjk = rjk/(phihat*(n-p));
   Rall = rjk+diag(1-diag(rjk));   % Correlation matrix
   for i=1:n;
       nii = ni(i);
       hui = huij(nstart(i):nstart(i)+nii-1,1);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       Xi = X(nstart(i):nstart(i)+nii-1,:);
       Ri = Rall(1:nii,1:nii);
       if max(max(isnan(Ri)))+max(max(isinf(Ri)))==1
           Ri = eye(nii);
           flag = 0;
       end
       if min(eig(Ri))<=0
           Ri = eye(nii);
           flag = 0;
       end
       SSi = chol(inv(Ri));
       Ai = diag(hui);
       Z1 = [Z1; SSi*zi];
       Z2 = [Z2; SSi*Ai*Xi];
       SS(nstart(i):nstart(i)+nii-1,1:nii) = SSi';
    end;
   
  case {6,'ind'};
    CorrStruct=6;
    alphat=1;
    
  otherwise
    disp('ERROR: Invalid correlation structure specified');
    return;
  end;

  % Estimate regression coefficients;
  if CorrStruct~=6;
    gammahat = regress(Z1,Z2);
    wtol = gammahat'*gammahat;
    bhat = bhat + gammahat;
  else
    wtol=0;    
  end;

  iter = iter+1;
end;


switch lower(Family);
case {1,'n'}
    sum1 = 0;
    sum2 = 0;
    for i=1:n;
       nii = ni(i);
       SSi = SS(nstart(i):nstart(i)+nii-1,1:nii);
       zi = z(nstart(i):nstart(i)+nii-1,1);
       temp = SSi*zi;
       sum1 = sum1 + 1/nii*temp'*temp;
       sum2 = sum2 + 1/nii*zi'*zi;
    end;
	tau = min(sum1,sum2)/n;
    
case {2,'b'};
    tau = 1;
case {3,'p'};
    tau = 1;
otherwise
end;
cumni  = cumsum(ni);
R = zeros(length(y),length(y));
for k=1:n
    idni =(cumni(k)-ni(k)+1):cumni(k);
    if CorrStruct~=6
        R(idni,idni) = Ri;
    else
        Ri           = eye(ni(k));
        R(idni,idni) = Ri;
    end
end
if CorrStruct~=6
    nwa = zeros(p,p);
    newb = zeros(p,p);
    for i=1:n
        nii = ni(i);
        Z1i = Z1(nstart(i):nstart(i)+nii-1,:);
        Z2i = Z2(nstart(i):nstart(i)+nii-1,:);
        nwa = nwa + Z2i'*Z2i;
        temp = Z2i'*Z1i;
        newb = newb + temp*temp';
    end;
    temp = inv(nwa);
    robcov = temp*newb*temp;    % Robust covariance matrix;
    switch lower(Family);
        case {1,'n'}
            QI = sum((y - X*bhat).^2)/tau;
            hA = eye(length(y));
        case {2,'b'};
            hmu = exp(X*bhat)./(1 + exp(X*bhat));
            hA = diag(hmu.*(1-hmu));
            QI = sum(y.*log(hmu./(1 - hmu)) + log(1 - hmu));
        case {3,'p'};
            hmu = exp(X*bhat);
            hA = diag(hmu);
            QI = sum(y.*log(hmu) -hmu);
        otherwise
    end;
    hrho_imori = trace(newb*temp);
    QICimori = -2*QI + 2*hrho_imori;
    hrho_pan = trace(robcov*X'*hA*X);
    QICpan   = -2*QI + 2*hrho_pan;
else
    switch lower(Family);
        case {1,'n'}
            QI = sum((y - X*bhat).^2)/tau;
            hA = eye(length(y));
        case {2,'b'};
            hmu = exp(X*bhat)./(1 + exp(X*bhat));
            hA = diag(hmu.*(1-hmu));
            QI = sum(y.*log(hmu./(1 - hmu)) + log(1 - hmu));
        case {3,'p'};
            hmu = exp(X*bhat);
            hA = diag(hmu);
            QI = sum(y.*log(hmu) -hmu);
        otherwise
    end;
    [ni,n] = subCounti(id);% ni contains cluster size, n is the culster number
    ni     = ni';
    cumni  = cumsum(ni);
    N      = length(y);
    VV     = zeros(N,N);
    for k = 1:n
        idni =(cumni(k)-ni(k)+1):cumni(k);
        VV(idni,idni) = (y(idni) - hmu(idni))*(y(idni) - hmu(idni))';
    end
    hrho_imori = trace((X'*hA*X)\(X'*VV*X));
    QICimori = -2*QI + 2*hrho_imori;
    hrho_pan =  hrho_imori;
    QICpan   = -2*QI + 2*hrho_pan;
end
end
function [ni,n] = subCounti(x);
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
% copywrite Sarah Ratcliffe 2000
%
% COMMAND:  [ni,n] = counti(x)
% ACTION:  For a vector of non-unique elements, x,
%          returns the number of times each unique
%          element is contained in x in ni. The
%          number of unique elements is optionally
%          returned in n.
%    Note: the non-unique elements of x must be 
%          grouped together and ni will contain in
%          numbers of each non-unique element in
%          the order in which they appear in x.
%~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

if nargin<1
  disp('ERROR: missing input variable');
  return;
end;

[n,p] = size(x);
if (n>1) & (p>1)
  disp('ERROR: x must be a vector');
  return;
end;

%% Initial Values
p = n*p;	% Number of elements in x
ni(1) = 1;	% First subject has one element
n = 1;		% One unique subject to start
xi = x(1);	% First unique subject = first element in x

for i=2:p    % cycle through elements counting as we go
  if x(i)==xi;
    ni(n) = ni(n)+1;
  else
    n = n+1;
    ni(n) = 1;
    xi = x(i);
  end
end;
end